@@ -20,11 +20,9 @@ import getpass
 import io
 import logging
 import os
-import sys
 
 from novaclient import api_versions
 from novaclient.v2 import servers
-from openstack import exceptions as sdk_exceptions
 from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
@@ -123,17 +121,21 @@ def _prefix_checked_value(prefix):
     return func
 
 
-def _prep_server_detail(compute_client, image_client, server):
+def _prep_server_detail(compute_client, image_client, server, refresh=True):
     """Prepare the detailed server dict for printing
 
     :param compute_client: a compute client instance
+    :param image_client: an image client instance
     :param server: a Server resource
+    :param refresh: Flag indicating if ``server`` is already the latest version
+                    or if it needs to be refreshed, for example when showing
+                    the latest details of a server after creating it.
     :rtype: a dict of server details
     """
-    info = server._info.copy()
-
-    server = utils.find_resource(compute_client.servers, info['id'])
-    info.update(server._info)
+    info = server.to_dict()
+    if refresh:
+        server = utils.find_resource(compute_client.servers, info['id'])
+        info.update(server.to_dict())
 
     # Convert the image blob to a name
     image_info = info.get('image', {})
@@ -147,12 +149,18 @@ def _prep_server_detail(compute_client, image_client, server):
 
     # Convert the flavor blob to a name
     flavor_info = info.get('flavor', {})
-    flavor_id = flavor_info.get('id', '')
-    try:
-        flavor = utils.find_resource(compute_client.flavors, flavor_id)
-        info['flavor'] = "%s (%s)" % (flavor.name, flavor_id)
-    except Exception:
-        info['flavor'] = flavor_id
+    # Microversion 2.47 puts the embedded flavor into the server response
+    # body but omits the id, so if not present we just expose the flavor
+    # dict in the server output.
+    if 'id' in flavor_info:
+        flavor_id = flavor_info.get('id', '')
+        try:
+            flavor = utils.find_resource(compute_client.flavors, flavor_id)
+            info['flavor'] = "%s (%s)" % (flavor.name, flavor_id)
+        except Exception:
+            info['flavor'] = flavor_id
+    else:
+        info['flavor'] = utils.format_dict(flavor_info)
 
     if 'os-extended-volumes:volumes_attached' in info:
         info.update(
@@ -181,7 +189,7 @@ def _prep_server_detail(compute_client, image_client, server):
     if 'tenant_id' in info:
         info['project_id'] = info.pop('tenant_id')
 
-    # Map power state num to meanful string
+    # Map power state num to meaningful string
     if 'OS-EXT-STS:power_state' in info:
         info['OS-EXT-STS:power_state'] = _format_servers_list_power_state(
             info['OS-EXT-STS:power_state'])
@@ -192,12 +200,6 @@ def _prep_server_detail(compute_client, image_client, server):
     return info
 
 
-def _show_progress(progress):
-    if progress:
-        sys.stdout.write('\rProgress: %s' % progress)
-        sys.stdout.flush()
-
-
 class AddFixedIP(command.Command):
     _description = _("Add fixed IP address to server")
 
@@ -249,16 +251,13 @@ class AddFloatingIP(network_common.NetworkAndComputeCommand):
         parser.add_argument(
             "ip_address",
             metavar="<ip-address>",
-            help=_("Floating IP address to assign to the first available "
-                   "server port (IP only)"),
+            help=_("Floating IP address to assign to server (IP only)"),
         )
         parser.add_argument(
             "--fixed-ip-address",
             metavar="<ip-address>",
             help=_(
-                "Fixed IP address to associate with this floating IP address. "
-                "The first server port containing the fixed IP address will "
-                "be used"
+                "Fixed IP address to associate with this floating IP address"
             ),
         )
         return parser
@@ -275,45 +274,12 @@ class AddFloatingIP(network_common.NetworkAndComputeCommand):
             compute_client.servers,
             parsed_args.server,
         )
-        ports = list(client.ports(device_id=server.id))
-        # If the fixed IP address was specified, we need to find the
-        # corresponding port.
+        port = list(client.ports(device_id=server.id))[0]
+        attrs['port_id'] = port.id
         if parsed_args.fixed_ip_address:
-            fip_address = parsed_args.fixed_ip_address
-            attrs['fixed_ip_address'] = fip_address
-            for port in ports:
-                for ip in port.fixed_ips:
-                    if ip['ip_address'] == fip_address:
-                        attrs['port_id'] = port.id
-                        break
-                else:
-                    continue
-                break
-            if 'port_id' not in attrs:
-                msg = _('No port found for fixed IP address %s')
-                raise exceptions.CommandError(msg % fip_address)
-            client.update_ip(obj, **attrs)
-        else:
-            # It's possible that one or more ports are not connected to a
-            # router and thus could fail association with a floating IP.
-            # Try each port until one succeeds. If none succeed, re-raise the
-            # last exception.
-            error = None
-            for port in ports:
-                attrs['port_id'] = port.id
-                try:
-                    client.update_ip(obj, **attrs)
-                except sdk_exceptions.NotFoundException as exp:
-                    # 404 ExternalGatewayForFloatingIPNotFound from neutron
-                    LOG.info('Skipped port %s because it is not attached to '
-                             'an external gateway', port.id)
-                    error = exp
-                    continue
-                else:
-                    error = None
-                    break
-            if error:
-                raise error
+            attrs['fixed_ip_address'] = parsed_args.fixed_ip_address
+
+        client.update_ip(obj, **attrs)
 
     def take_action_compute(self, client, parsed_args):
         client.api.floating_ip_add(
@@ -480,6 +446,12 @@ class CreateServer(command.ShowOne):
             metavar='<image>',
             help=_('Create server boot disk from this image (name or ID)'),
         )
+        disk_group.add_argument(
+            '--image-property',
+            metavar='<key=value>',
+            action=parseractions.KeyValueAction,
+            help=_("Image property to be matched"),
+        )
         disk_group.add_argument(
             '--volume',
             metavar='<volume>',
@@ -636,6 +608,12 @@ class CreateServer(command.ShowOne):
         return parser
 
     def take_action(self, parsed_args):
+
+        def _show_progress(progress):
+            if progress:
+                self.app.stdout.write('\rProgress: %s' % progress)
+                self.app.stdout.flush()
+
         compute_client = self.app.client_manager.compute
         volume_client = self.app.client_manager.volume
         image_client = self.app.client_manager.image
@@ -648,6 +626,45 @@ class CreateServer(command.ShowOne):
                 parsed_args.image,
             )
 
+        if not image and parsed_args.image_property:
+            def emit_duplicated_warning(img, image_property):
+                img_uuid_list = [str(image.id) for image in img]
+                LOG.warning(_('Multiple matching images: %(img_uuid_list)s\n'
+                              'Using image: %(chosen_one)s') %
+                            {'img_uuid_list': img_uuid_list,
+                             'chosen_one': img_uuid_list[0]})
+
+            def _match_image(image_api, wanted_properties):
+                image_list = image_api.image_list()
+                images_matched = []
+                for img in image_list:
+                    img_dict = {}
+                    # exclude any unhashable entries
+                    for key, value in img.items():
+                        try:
+                            set([key, value])
+                        except TypeError:
+                            pass
+                        else:
+                            img_dict[key] = value
+                    if all(k in img_dict and img_dict[k] == v
+                           for k, v in wanted_properties.items()):
+                        images_matched.append(img)
+                    else:
+                        return []
+                return images_matched
+
+            images = _match_image(image_client.api, parsed_args.image_property)
+            if len(images) > 1:
+                emit_duplicated_warning(images,
+                                        parsed_args.image_property)
+            if images:
+                image = images[0]
+            else:
+                raise exceptions.CommandError(_("No images match the "
+                                                "property expected by "
+                                                "--image-property"))
+
         # Lookup parsed_args.volume
         volume = None
         if parsed_args.volume:
@@ -792,14 +809,9 @@ class CreateServer(command.ShowOne):
                     raise exceptions.CommandError(msg)
                 nics = nics[0]
         else:
-            # Compute API version >= 2.37 requires a value, so default to
-            # 'auto' to maintain legacy behavior if a nic wasn't specified.
-            if compute_client.api_version >= api_versions.APIVersion('2.37'):
-                nics = 'auto'
-            else:
-                # Default to empty list if nothing was specified, let nova
-                # side to decide the default behavior.
-                nics = []
+            # Default to empty list if nothing was specified, let nova side to
+            # decide the default behavior.
+            nics = []
 
         # Check security group exist and convert ID to name
         security_group_names = []
@@ -875,11 +887,11 @@ class CreateServer(command.ShowOne):
                 server.id,
                 callback=_show_progress,
             ):
-                sys.stdout.write('\n')
+                self.app.stdout.write('\n')
             else:
                 LOG.error(_('Error creating server: %s'),
                           parsed_args.server_name)
-                sys.stdout.write(_('Error creating server\n'))
+                self.app.stdout.write(_('Error creating server\n'))
                 raise SystemExit
 
         details = _prep_server_detail(compute_client, image_client, server)
@@ -933,6 +945,12 @@ class DeleteServer(command.Command):
         return parser
 
     def take_action(self, parsed_args):
+
+        def _show_progress(progress):
+            if progress:
+                self.app.stdout.write('\rProgress: %s' % progress)
+                self.app.stdout.flush()
+
         compute_client = self.app.client_manager.compute
         for server in parsed_args.server:
             server_obj = utils.find_resource(
@@ -944,11 +962,11 @@ class DeleteServer(command.Command):
                     server_obj.id,
                     callback=_show_progress,
                 ):
-                    sys.stdout.write('\n')
+                    self.app.stdout.write('\n')
                 else:
                     LOG.error(_('Error deleting server: %s'),
                               server_obj.id)
-                    sys.stdout.write(_('Error deleting server\n'))
+                    self.app.stdout.write(_('Error deleting server\n'))
                     raise SystemExit
 
 
@@ -1209,7 +1227,7 @@ class ListServer(command.Lister):
         # Create a dict that maps image_id to image object.
         # Needed so that we can display the "Image Name" column.
         # "Image Name" is not crucial, so we swallow any exceptions.
-        if not parsed_args.no_name_lookup:
+        if data and not parsed_args.no_name_lookup:
             try:
                 images_list = self.app.client_manager.image.images.list()
                 for i in images_list:
@@ -1221,9 +1239,9 @@ class ListServer(command.Lister):
         # Create a dict that maps flavor_id to flavor object.
         # Needed so that we can display the "Flavor Name" column.
         # "Flavor Name" is not crucial, so we swallow any exceptions.
-        if not parsed_args.no_name_lookup:
+        if data and not parsed_args.no_name_lookup:
             try:
-                flavors_list = compute_client.flavors.list()
+                flavors_list = compute_client.flavors.list(is_public=None)
                 for i in flavors_list:
                     flavors[i.id] = i
             except Exception:
@@ -1246,6 +1264,10 @@ class ListServer(command.Lister):
                     s.flavor_name = flavor.name
                 s.flavor_id = s.flavor['id']
             else:
+                # TODO(mriedem): Fix this for microversion >= 2.47 where the
+                # flavor is embedded in the server response without the id.
+                # We likely need to drop the Flavor ID column in that case if
+                # --long is specified.
                 s.flavor_name = ''
                 s.flavor_id = ''
 
@@ -1351,6 +1373,11 @@ class MigrateServer(command.Command):
 
     def take_action(self, parsed_args):
 
+        def _show_progress(progress):
+            if progress:
+                self.app.stdout.write('\rProgress: %s' % progress)
+                self.app.stdout.flush()
+
         compute_client = self.app.client_manager.compute
 
         server = utils.find_resource(
@@ -1379,11 +1406,11 @@ class MigrateServer(command.Command):
                 success_status=['active', 'verify_resize'],
                 callback=_show_progress,
             ):
-                sys.stdout.write(_('Complete\n'))
+                self.app.stdout.write(_('Complete\n'))
             else:
                 LOG.error(_('Error migrating server: %s'),
                           server.id)
-                sys.stdout.write(_('Error migrating server\n'))
+                self.app.stdout.write(_('Error migrating server\n'))
                 raise SystemExit
 
 
@@ -1444,6 +1471,12 @@ class RebootServer(command.Command):
         return parser
 
     def take_action(self, parsed_args):
+
+        def _show_progress(progress):
+            if progress:
+                self.app.stdout.write('\rProgress: %s' % progress)
+                self.app.stdout.flush()
+
         compute_client = self.app.client_manager.compute
         server = utils.find_resource(
             compute_client.servers, parsed_args.server)
@@ -1455,11 +1488,11 @@ class RebootServer(command.Command):
                 server.id,
                 callback=_show_progress,
             ):
-                sys.stdout.write(_('Complete\n'))
+                self.app.stdout.write(_('Complete\n'))
             else:
                 LOG.error(_('Error rebooting server: %s'),
                           server.id)
-                sys.stdout.write(_('Error rebooting server\n'))
+                self.app.stdout.write(_('Error rebooting server\n'))
                 raise SystemExit
 
 
@@ -1492,6 +1525,12 @@ class RebuildServer(command.ShowOne):
         return parser
 
     def take_action(self, parsed_args):
+
+        def _show_progress(progress):
+            if progress:
+                self.app.stdout.write('\rProgress: %s' % progress)
+                self.app.stdout.flush()
+
         compute_client = self.app.client_manager.compute
         image_client = self.app.client_manager.image
 
@@ -1499,7 +1538,8 @@ class RebuildServer(command.ShowOne):
             compute_client.servers, parsed_args.server)
 
         # If parsed_args.image is not set, default to the currently used one.
-        image_id = parsed_args.image or server._info.get('image', {}).get('id')
+        image_id = parsed_args.image or server.to_dict().get(
+            'image', {}).get('id')
         image = utils.find_resource(image_client.images, image_id)
 
         server = server.rebuild(image, parsed_args.password)
@@ -1509,14 +1549,15 @@ class RebuildServer(command.ShowOne):
                 server.id,
                 callback=_show_progress,
             ):
-                sys.stdout.write(_('Complete\n'))
+                self.app.stdout.write(_('Complete\n'))
             else:
                 LOG.error(_('Error rebuilding server: %s'),
                           server.id)
-                sys.stdout.write(_('Error rebuilding server\n'))
+                self.app.stdout.write(_('Error rebuilding server\n'))
                 raise SystemExit
 
-        details = _prep_server_detail(compute_client, image_client, server)
+        details = _prep_server_detail(compute_client, image_client, server,
+                                      refresh=False)
         return zip(*sorted(six.iteritems(details)))
 
 
@@ -1798,6 +1839,11 @@ the new server and restart the old one.""")
 
     def take_action(self, parsed_args):
 
+        def _show_progress(progress):
+            if progress:
+                self.app.stdout.write('\rProgress: %s' % progress)
+                self.app.stdout.flush()
+
         compute_client = self.app.client_manager.compute
         server = utils.find_resource(
             compute_client.servers,
@@ -1816,11 +1862,11 @@ the new server and restart the old one.""")
                     success_status=['active', 'verify_resize'],
                     callback=_show_progress,
                 ):
-                    sys.stdout.write(_('Complete\n'))
+                    self.app.stdout.write(_('Complete\n'))
                 else:
                     LOG.error(_('Error resizing server: %s'),
                               server.id)
-                    sys.stdout.write(_('Error resizing server\n'))
+                    self.app.stdout.write(_('Error resizing server\n'))
                     raise SystemExit
         elif parsed_args.confirm:
             compute_client.servers.confirm_resize(server)
@@ -1961,7 +2007,9 @@ class ShelveServer(command.Command):
 
 
 class ShowServer(command.ShowOne):
-    _description = _("Show server details")
+    _description = _(
+        "Show server details. Specify ``--os-compute-api-version 2.47`` "
+        "or higher to see the embedded flavor information for the server.")
 
     def get_parser(self, prog_name):
         parser = super(ShowServer, self).get_parser(prog_name)
@@ -1986,11 +2034,14 @@ class ShowServer(command.ShowOne):
         if parsed_args.diagnostics:
             (resp, data) = server.diagnostics()
             if not resp.status_code == 200:
-                sys.stderr.write(_("Error retrieving diagnostics data\n"))
+                self.app.stderr.write(_(
+                    "Error retrieving diagnostics data\n"
+                ))
                 return ({}, {})
         else:
             data = _prep_server_detail(compute_client,
-                                       self.app.client_manager.image, server)
+                                       self.app.client_manager.image, server,
+                                       refresh=False)
 
         return zip(*sorted(six.iteritems(data)))
 
