@@ -1046,6 +1046,36 @@ class TestSetSubnet(TestSubnet):
             _testsubnet, **attrs)
         self.assertIsNone(result)
 
+    def test_clear_options(self):
+        _testsubnet = network_fakes.FakeSubnet.create_one_subnet(
+            {'host_routes': [{'destination': '10.20.20.0/24',
+                              'nexthop': '10.20.20.1'}],
+             'allocation_pools': [{'start': '8.8.8.200',
+                                   'end': '8.8.8.250'}],
+             'dns_nameservers': ['10.0.0.1'], })
+        self.network.find_subnet = mock.Mock(return_value=_testsubnet)
+        arglist = [
+            '--no-host-route',
+            '--no-allocation-pool',
+            '--no-dns-nameservers',
+            _testsubnet.name,
+        ]
+        verifylist = [
+            ('no_dns_nameservers', True),
+            ('no_host_route', True),
+            ('no_allocation_pool', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        attrs = {
+            'host_routes': [],
+            'allocation_pools': [],
+            'dns_nameservers': [],
+        }
+        self.network.update_subnet.assert_called_once_with(
+            _testsubnet, **attrs)
+        self.assertIsNone(result)
+
     def _test_set_tags(self, with_tags=True):
         if with_tags:
             arglist = ['--tag', 'red', '--tag', 'blue']
@@ -1074,6 +1104,29 @@ class TestSetSubnet(TestSubnet):
     def test_set_with_no_tag(self):
         self._test_set_tags(with_tags=False)
 
+    def test_set_segment(self):
+        _net = network_fakes.FakeNetwork.create_one_network()
+        _segment = network_fakes.FakeNetworkSegment.create_one_network_segment(
+            attrs={'network_id': _net.id})
+        _subnet = network_fakes.FakeSubnet.create_one_subnet(
+            {'host_routes': [{'destination': '10.20.20.0/24',
+                              'nexthop': '10.20.20.1'}],
+             'allocation_pools': [{'start': '8.8.8.200',
+                                   'end': '8.8.8.250'}],
+             'dns_nameservers': ["10.0.0.1"],
+             'network_id': _net.id,
+             'segment_id': None})
+        self.network.find_subnet = mock.Mock(return_value=_subnet)
+        self.network.find_segment = mock.Mock(return_value=_segment)
+        arglist = ['--network-segment', _segment.id, _subnet.name]
+        verifylist = [('network_segment', _segment.id)]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+        attrs = {'segment_id': _segment.id}
+        self.network.update_subnet.assert_called_once_with(_subnet, **attrs)
+        self.network.update_subnet.assert_called_with(_subnet, **attrs)
+        self.assertIsNone(result)
+
 
 class TestShowSubnet(TestSubnet):
     # The subnets to be shown
